@@ -44,7 +44,9 @@
 */
 #include "isotree.hpp"
 
-/*  Fit Isolation Forest (or variant) model 
+bool interrupt_switch;
+
+/*  Fit Isolation Forest model, or variant of it such as SCiForest
 * 
 * Parameters:
 * ===========
@@ -253,6 +255,12 @@
 *       systems, this means no column can have more than 20 different categories if using 'all_perm=true',
 *       but note that this is not checked within the function.
 *       Ignored when not using categorical variables or not doing splits by pooled gain or using 'ndim>1'.
+* - coef_by_prop
+*       In the extended model, whether to sort the randomly-generated coefficients for categories
+*       according to their relative frequency in the tree node. This might provide better results when using
+*       categorical variables with too many categories, but is not recommended, and not reflective of
+*       real "categorical-ness". Ignored for the regular model ('ndim=1') and/or when not using categorical
+*       variables.
 * - imputer (out)
 *       Pointer to already-allocated imputer object, which can be used to produce missing value imputations
 *       in new data. Pass NULL if no missing value imputations are required. Note that this is not related to
@@ -284,6 +292,15 @@
 *       allocated, even if the thread does not end up being used. Ignored when not building with
 *       OpenMP support.
 * 
+* Returns
+* =======
+* Will return macro 'EXIT_SUCCESS' (typically =0) upon completion.
+* If the process receives an interrupt signal, will return instead
+* 'EXIT_FAILURE' (typically =1). If you do not have any way of determining
+* what these values correspond to, you can use the functions
+* 'return_EXIT_SUCESS' and 'return_EXIT_FAILURE', which will return them
+* as integers.
+* 
 * References
 * ==========
 * [1] Liu, Fei Tony, Kai Ming Ting, and Zhi-Hua Zhou.
@@ -307,7 +324,7 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
                 double numeric_data[],  size_t ncols_numeric,
                 int    categ_data[],    size_t ncols_categ,    int ncat[],
                 double Xc[], sparse_ix Xc_ind[], sparse_ix Xc_indptr[],
-                size_t ndim, size_t ntry, CoefType coef_type,
+                size_t ndim, size_t ntry, CoefType coef_type, bool coef_by_prop,
                 double sample_weights[], bool with_replacement, bool weight_as_sample,
                 size_t nrows, size_t sample_size, size_t ntrees, size_t max_depth,
                 bool   limit_depth, bool penalize_range,
@@ -346,7 +363,7 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
                                 prob_pick_by_gain_pl,  (model_outputs == NULL)? 0 : prob_split_by_gain_pl,
                                 min_gain, cat_split_type, new_cat_action, missing_action, all_perm,
                                 (model_outputs != NULL)? 0 : ndim, (model_outputs != NULL)? 0 : ntry,
-                                coef_type, calc_dist, (bool)(output_depths != NULL), impute_at_fit,
+                                coef_type, coef_by_prop, calc_dist, (bool)(output_depths != NULL), impute_at_fit,
                                 depth_imp, weigh_imp_rows, min_imp_obs};
 
     /* if using weights as sampling probability, build a binary tree for faster sampling */
@@ -366,6 +383,7 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
     if (model_outputs != NULL)
     {
         model_outputs->trees.resize(ntrees);
+        model_outputs->trees.shrink_to_fit();
         model_outputs->new_cat_action = new_cat_action;
         model_outputs->cat_split_type = cat_split_type;
         model_outputs->missing_action = missing_action;
@@ -377,6 +395,7 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
     else
     {
         model_outputs_ext->hplanes.resize(ntrees);
+        model_outputs_ext->hplanes.shrink_to_fit();
         model_outputs_ext->new_cat_action = new_cat_action;
         model_outputs_ext->cat_split_type = cat_split_type;
         model_outputs_ext->missing_action = missing_action;
@@ -397,10 +416,16 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
         std::vector<WorkerMemory> worker_memory(1);
     #endif
 
+    /* Global variable that determines if the procedure receives a stop signal */
+    interrupt_switch = false;
+
     /* grow trees */
     #pragma omp parallel for num_threads(nthreads) schedule(dynamic) shared(model_outputs, model_outputs_ext, worker_memory, input_data, model_params)
     for (size_t_for tree = 0; tree < ntrees; tree++)
     {
+        if (interrupt_switch)
+            continue; /* Cannot break with OpenMP==2.0 (MSVC) */
+
         if (
             model_params.impute_at_fit &&
             input_data.n_missing &&
@@ -436,14 +461,25 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
         else
             model_outputs_ext->hplanes[tree].shrink_to_fit();
 
+        signal(SIGINT, set_interrup_global_variable);
     }
 
+    /* check if the procedure got interrupted */
+    if (interrupt_switch) return EXIT_FAILURE;
+    interrupt_switch = false;
+
+    if ((model_outputs != NULL))
+        model_outputs->trees.shrink_to_fit();
+    else
+        model_outputs_ext->hplanes.shrink_to_fit();
+
     /* if calculating similarity/distance, now need to reduce and average */
     if (calc_dist)
         gather_sim_result(NULL, &worker_memory,
                           NULL, &input_data,
                           model_outputs, model_outputs_ext,
-                          tmat, model_params.ntrees, false,
+                          tmat, NULL, 0,
+                          model_params.ntrees, false,
                           standardize_dist, nthreads);
 
     /* same for depths */
@@ -538,9 +574,10 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
 *       Pass NULL if there are no categorical columns. The encoding must be the same as was used
 *       in the data to which the model was fit.
 *       Each category should be represented as an integer, and these integers must start at zero and
-*       be in consecutive order - i.e. if category '3' is present, category '2' must also be present
-*       (note that they are not treated as being ordinal, this is just an encoding). Missing values
-*       should be encoded as negative numbers such as (-1).
+*       be in consecutive order - i.e. if category '3' is present, category '2' must have also been
+*       present when the model was fit (note that they are not treated as being ordinal, this is just
+*       an encoding). Missing values should be encoded as negative numbers such as (-1). The encoding
+*       must be the same as was used in the data to which the model was fit.
 *       If the model from 'fit_iforest' was fit to categorical data, must pass categorical data with the same number
 *       of columns and the same category encoding.
 * - ncols_categ
@@ -627,6 +664,9 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
 * - all_perm
 *       Same parameter as for 'fit_iforest' (see the documentation in there for details). Can be changed from
 *       what was originally passed to 'fit_iforest'.
+* - coef_by_prop
+*       Same parameter as for 'fit_iforest' (see the documentation in there for details). Can be changed from
+*       what was originally passed to 'fit_iforest'.
 * - impute_nodes
 *       Pointer to already-allocated imputation nodes for the tree that will be built. Note that the number of
 *       entries in the imputation object must match the number of fitted trees when it is used.  Pass
@@ -641,7 +681,7 @@ int add_tree(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
              double numeric_data[],  size_t ncols_numeric,
              int    categ_data[],    size_t ncols_categ,    int ncat[],
              double Xc[], sparse_ix Xc_ind[], sparse_ix Xc_indptr[],
-             size_t ndim, size_t ntry, CoefType coef_type,
+             size_t ndim, size_t ntry, CoefType coef_type, bool coef_by_prop,
              double sample_weights[], size_t nrows, size_t max_depth,
              bool   limit_depth,   bool penalize_range,
              double col_weights[], bool weigh_by_kurt,
@@ -670,7 +710,7 @@ int add_tree(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
                                 prob_pick_by_gain_pl,  (model_outputs == NULL)? 0 : prob_split_by_gain_pl,
                                 min_gain, cat_split_type, new_cat_action, missing_action, all_perm,
                                 (model_outputs != NULL)? 0 : ndim, (model_outputs != NULL)? 0 : ntry,
-                                coef_type, false, false, false, depth_imp, weigh_imp_rows, min_imp_obs};
+                                coef_type, coef_by_prop, false, false, false, depth_imp, weigh_imp_rows, min_imp_obs};
 
     std::unique_ptr<WorkerMemory> workspace = std::unique_ptr<WorkerMemory>(new WorkerMemory);
 
