@@ -1,6 +1,3 @@
-#include "isotree.h"
-
-
 /*    Isolation forests and variations thereof, with adjustments for incorporation
 *     of categorical variables and missing values.
 *     Writen for C++11 standard and aimed at being used in R and Python.
@@ -45,6 +42,7 @@
 *     OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *     OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
+#include "isotree.h"
 
 /* Predict outlier score, average depth, or terminal node numbers
 * 
@@ -62,10 +60,10 @@
 *       and the column order must be the same as in the data that was used to fit the model.
 *       Pass NULL if there are no categorical columns.
 *       Each category should be represented as an integer, and these integers must start at zero and
-*       be in consecutive order - i.e. if category '3' is present, category '2' must also be present
-*       (note that they are not treated as being ordinal, this is just an encoding). Missing values
-*       should be encoded as negative numbers such as (-1). The encoding must be the same as was used
-*       in the data to which the model was fit.
+*       be in consecutive order - i.e. if category '3' is present, category '2' must have also been
+*       present when the model was fit (note that they are not treated as being ordinal, this is just
+*       an encoding). Missing values should be encoded as negative numbers such as (-1). The encoding
+*       must be the same as was used in the data to which the model was fit.
 * - Xc[nnz]
 *       Pointer to numeric data in sparse numeric matrix in CSC format (column-compressed).
 *       Pass NULL if there are no sparse numeric columns.
@@ -391,7 +389,7 @@ double traverse_itree(std::vector<IsoTree>     &tree,
 
     while (true)
     {
-        if (tree[curr_lev].score > 0)
+        if (tree[curr_lev].score >= 0.)
         {
             if (tree_num != NULL)
                 tree_num[row] = curr_lev;
